// Copyright (C) 2007-2009 David Kment
// For license see license.txt

#ifndef _CIMGARCHIVELOADER_H_
#define _CIMGARCHIVELOADER_H_

#include "irrlicht.h"
#include "IFileSystem.h"

using namespace irr;
using namespace core;
using namespace io;

struct SIMGFileHeaderV2
{
    c8 fourcc[4];
    u32 entries;
};

struct SIMGDirEntry
{
    u32 pos;
    u32 length;
    c8 name[24];
};

struct SIMGFileEntry: public IFileArchiveEntry
{
    core::string<c16> simpleFileName;
    u32 pos;
    u32 length;
};

class CIMGArchiveLoader : public IArchiveLoader
{
public:

    CIMGArchiveLoader(io::IFileSystem* fs);
    virtual ~CIMGArchiveLoader();
    virtual bool isALoadableFileFormat(const core::string<c16>& filename) const;
    virtual IFileArchive* createArchive(const core::string<c16>& filename, bool ignoreCase, bool ignorePaths) const;
    virtual bool isALoadableFileFormat(io::IReadFile* file) const;
    virtual io::IFileArchive* createArchive(io::IReadFile* file, bool ignoreCase, bool ignorePaths) const;

private:
    io::IFileSystem* FileSystem;
};


class CIMGReader : public IFileArchive
{
public:

    CIMGReader(IReadFile* file, bool ignoreCase, bool ignorePaths);
    virtual ~CIMGReader();
    void init();
    virtual IReadFile* openFile(const core::string<c16>& filename);
    IReadFile* openFile(s32 index);
	virtual u32 getFileCount() const;
    virtual const IFileArchiveEntry* getFileInfo(u32 index);
    virtual s32 findFile(const core::string<c16>& filename);
    virtual const core::string<c16>& getArchiveType() {return Type;}
    virtual const core::string<c16>& getArchiveName () {return File->getFileName();}
    void setFileSystem(io::IFileSystem* fs);

private:
    core::string<c16> Type;
    IReadFile* File;
    core::array<SIMGFileEntry> FileList;
    bool IgnoreCase;
    bool IgnorePaths;
    io::IFileSystem* FileSystem;
};

#endif /* _CIMGARCHIVELOADER_H_ */
