// Copyright (C) 2007-2009 David Kment
// For license see license.txt

#include "CRenderManager.h"
#include <vector>

CRenderManager::CRenderManager(CGTASceneNode *sceneNode)
{
    initialised = false;

    SceneNode = sceneNode;

    // for initialisation, disable rendering for all instances
    std::vector<CGTASceneNode::ObjectInstance> *instances = SceneNode->getObjectInstances();
    vector<CGTASceneNode::ObjectInstance>::iterator instance;
    for(instance = instances->begin(); instance != instances->end(); instance++)
    {
        instance->setRender(false);
    }
}

void CRenderManager::update(double x, double y)
{
    if(!initialised)
    {
        initialised = true;

        PositionX = x;
        PositionY = y;

        process();
    }

    double distanceX = x - PositionX;
    double distanceY = y - PositionY;

    if(distanceX < 0.0f) {distanceX *= -1.0f;}
    if(distanceY < 0.0f) {distanceY *= -1.0f;}

    if(distanceX > UNIT_DISTANCE_TRIGGER || distanceY > UNIT_DISTANCE_TRIGGER)
    {
        PositionX = x;
        PositionY = y;

        process();
    }
}

void CRenderManager::process()
{
    std::vector<CGTASceneNode::ObjectInstance> *instances = SceneNode->getObjectInstances();

    double distanceX, distanceY;

    vector<CGTASceneNode::ObjectInstance>::iterator instance;
    for(instance = instances->begin(); instance != instances->end(); instance++)
    {
        distanceX = instance->getPosition().X - PositionX;
        distanceY = instance->getPosition().Z - PositionY;

        if(distanceX < 0.0f) {distanceX *= -1.0f;}
        if(distanceY < 0.0f) {distanceY *= -1.0f;}

        if(distanceX < instance->getDrawDistance() && distanceY < instance->getDrawDistance())
        {
            // object is in visible distance

            if(instance->isLODMesh() && (distanceX < 300.0f || distanceY < 300.0f))
            {
                if(instance->getRender() == true)
                {
                    instance->setRender(false);
                }
            }
            else
            {
                if(instance->getRender() == false)
                {
                    instance->setRender(true);
                }
            }
        }
        else
        {
            if(instance->getRender() == true)
            {
                instance->setRender(false);
            }
        }
    }
}
