// Copyright (C) 2007-2009 David Kment
// For license see license.txt

#ifndef _CTXDARCHIVELOADER_H_
#define _CTXDARCHIVELOADER_H_

#include "irrlicht.h"
#include "IFileSystem.h"

// DFF section IDs
#define DFF_RW_DATA 1
#define DFF_RW_EXTENSION 3
#define DFF_RW_TEXTURE_NATIVE 21
#define DFF_RW_DICTIONARY 22

using namespace irr;
using namespace core;
using namespace io;

struct STXDTextureHeader
{
    u8 ignore[8];
    u8 diffuseTextureName[32];
    u8 alphaTextureName[32];
};

struct STXDFileEntry: public IFileArchiveEntry
{
    core::string<c16> simpleFileName;
    u32 pos;
    u32 length;
};

class CTXDArchiveLoader : public IArchiveLoader
{
public:

    CTXDArchiveLoader(io::IFileSystem* fs);
    virtual ~CTXDArchiveLoader();
    virtual bool isALoadableFileFormat(const core::string<c16>& filename) const;
    virtual IFileArchive* createArchive(const core::string<c16>& filename, bool ignoreCase, bool ignorePaths) const;
    virtual bool isALoadableFileFormat(io::IReadFile* file) const;
    virtual io::IFileArchive* createArchive(io::IReadFile* file, bool ignoreCase, bool ignorePaths) const;

private:
    io::IFileSystem* FileSystem;
    core::array< core::string<c16> >* LoadedList;
};


class CTXDReader : public IFileArchive
{
public:

    CTXDReader(IReadFile* file, bool ignoreCase, bool ignorePaths);
    virtual ~CTXDReader();
    virtual IReadFile* openFile(const core::string<c16>& filename);
    IReadFile* openFile(s32 index);
    virtual u32 getFileCount() const;
    virtual const IFileArchiveEntry* getFileInfo(u32 index);
    virtual s32 findFile(const core::string<c16>& filename);
    virtual const core::string<c16>& getArchiveType() {return Type;}
    virtual const core::string<c16>& getArchiveName () {return File->getFileName();}
    void setFileSystem(io::IFileSystem* fs);

private:
    core::string<c16> Type;
    core::string<c16> TXDName;
    IReadFile* File;
    core::array<STXDFileEntry> FileList;
    bool IgnoreCase;
    bool IgnorePaths;
    io::IFileSystem* FileSystem;

    void parseRWSection(int size, int depthlevel, int parentType);
    void processRWData(int size, int type, int version);
};

#endif /* _CTXDARCHIVELOADER_H_ */
