// Copyright (C) 2007-2009 David Kment
// For license see license.txt

#include "main.h"

#include <irrlicht.h>
#include <windows.h>

#include "CGTASceneNode.h"
#include "CDFFMeshFileLoader.h"
#include "CIMGArchiveLoader.h"
#include "CTXDArchiveLoader.h"
#include "CImageLoaderGTATexture.h"
#include "CEventReceiver.h"
#include "CRenderManager.h"

using namespace std;

using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

int main(int argc, char *argv[])
{
    int gameId = 0;

    core::string<c16> userWorkingDirectory = "";

    int i;
    for(i=1; i < argc; i++)
    {
        if(i==1)
        {
            userWorkingDirectory = argv[i];

            while(userWorkingDirectory.find("\"") != -1)
            {
                userWorkingDirectory.erase(userWorkingDirectory.find("\""));
            }
        }
    }

    CEventReceiver *receiver = new CEventReceiver();

    // use EDT_OPENGL or EDT_BURNINGSVIDEO
    IrrlichtDevice *device = createDevice(video::EDT_OPENGL, dimension2d<u32>(800, 600), 32, false, false, false, receiver);

    device->setWindowCaption(L"GTALight");
    device->getCursorControl()->setVisible(false);

    IVideoDriver *driver = device->getVideoDriver();
    ISceneManager *smgr = device->getSceneManager();
    IGUIEnvironment *guienv = device->getGUIEnvironment();
    IFileSystem *fs = device->getFileSystem();

    CIMGArchiveLoader imgloader = CIMGArchiveLoader(fs);
    fs->addArchiveLoader(&imgloader);

    CTXDArchiveLoader txdloader = CTXDArchiveLoader(fs);
    fs->addArchiveLoader(&txdloader);

    CDFFMeshFileLoader dffloader = CDFFMeshFileLoader(device);
    smgr->addExternalMeshLoader(&dffloader);

    CImageLoaderGTATexture gtatextureloader = CImageLoaderGTATexture(device);
    driver->addExternalImageLoader(&gtatextureloader);

    // smgr->getParameters()->setAttribute("GTA_LOD_MESHES_ONLY", lodMode);
    smgr->getParameters()->setAttribute("GTA_TEXTURES_PATH", "");
    smgr->getParameters()->setAttribute("GTA_MODELS_PATH", "");
    smgr->getParameters()->setAttribute("GTA_TEXTURES_EXTENSION", "gtatexture");

    if(userWorkingDirectory.size() > 0)
    {
        if(device->getFileSystem()->changeWorkingDirectoryTo(userWorkingDirectory))
        {
            printf("changing to user data directory: %s\n", userWorkingDirectory.c_str());
        }
        else
        {
            printf("error, invalid user data directory supplied: %s\n", userWorkingDirectory.c_str());
        }
    }

    // detect game ID
    if(fs->existFile("gta3.exe"))
    {
        gameId = GAMEID_GTA3;
    }
    else if(fs->existFile("gta-vc.exe"))
    {
        gameId = GAMEID_GTAVC;
    }
    else if(fs->existFile("gta_sa.exe"))
    {
        gameId = GAMEID_GTASA;
    }

    if(!fs->registerFileArchive("models\\gta3.img", true, false))
    {
        printf("error, cannot register archive models\\gta3.img\n");
    }

    CGTASceneNode *gtanode = new CGTASceneNode(device, gameId, NULL, smgr, -1, core::vector3df(0.0f, 0.0f, 0.0f), core::vector3df(0.0f, 0.0f, 0.0f), core::vector3df(1.0f, 1.0f, 1.0f));
    gtanode->initialise();

    printf("Worldmap initialised\n");

    ICameraSceneNode* camera = smgr->addCameraSceneNodeFPS(NULL, 50.0f, 0.2f, -1, 0, 0, false, 0.0f);

    CRenderManager* manager;

    if(DYNAMIC_LOD_MODE)
    {
        manager = new CRenderManager(gtanode);
    }

    int fps = 0;
    int trisdrawn;

    while(device->run())
    {
        if (device->isWindowActive())
        {
            if(DYNAMIC_LOD_MODE)
            {
                manager->update(camera->getAbsolutePosition().X, camera->getAbsolutePosition().Z);
            }

            camera->setFarValue((float)receiver->getFarValue());

            fps = driver->getFPS();
            trisdrawn = driver->getPrimitiveCountDrawn();

            driver->beginScene(true, true, SColor(255,100,101,140));

            smgr->drawAll();
            guienv->drawAll();

            driver->endScene();
        }
    }

    device->drop();

    return 0;
}
